# parsing.py
#
# by Arthur Kevin McGrath
#
"""The functions in this file parse either an Object Name or a Configuration ID.\n
Both functions look for values that match a given key.  What keys exist depends on the Object Name or a Configuration ID in question.  The burden is on the caller to know what keys exist.\n\n"""

# findValueInConfigurationIDString
# parameters:
#   idString - the value returned by AdminControl.getConfigID
#                 or one value out of the array of values returned
#                 by AdminConfig.list
#   p - the property whose value we want to search for
#      known legal values are:
#           "display name" for the name of this thing (if available - might be empty string)
#           "cell" for the cell name (note the singular)
#           "node" for the node name (note the singular)
#           "server" for the server name (note the singular)
#           "application" for the application EAR or WAR file name
# returns either the value or '"Not Found"
def findValueInConfigurationIDString( idString, p ):
  """
If you provide a configuration ID string and the name of a property to find
in that string, this method parses the configuration ID and 
returns the value of that property.
parameters:
  idString - the value returned by AdminControl.getConfigID
                or one value out of the array of values returned
                by AdminConfig.list
  p - the property whose value we want to search for
     known legal values are:
          "display name" for the name of this thing (if available - might be empty string)
          "cell" for the cell name (note the singular)
          "node" for the node name (note the singular)
          "server" for the server name (note the singular)
          "application" for the application EAR or WAR file name
returns either the value or '"Not Found"
  """
  # initialize some variables
  value = "Not Found"
  namelessString = "Some bogus value"
  startOfConfigID = "(cells/"
  s = []
  #       DEBUG ONLY!!!!!
  #print "# # # inside findValueInConfigurationIDString()"
  #print "     idString- - >" + idString + "< - -"
  #print "                p- - >" + p + "< - -"
  # a configurationID has a whole bunch of junk that gets in the way of parsing
  # eliminate that junk
  # step 1: split the idString 
  #            separate the name and the configuration ID into separate strings
  #print "idString.find( startOfConfigID )",idString.find( startOfConfigID )
  namelessString = idString[ idString.find( startOfConfigID ) : ]
  name = idString[   : idString.find( startOfConfigID )  ]
  #       DEBUG ONLY!!!!!
  #print "                  display name- - >" + name + "< - -"
  #print "namelessString- - >" + namelessString + "< - -"
  #       DEBUG ONLY!!!!!
  #print "\nidString[ ", idString.find( startOfConfigID ) , " : ] - ->" + namelessString + "<- -\n"
  #print "\nidString[ :",idString.find( startOfConfigID), " ] - ->" + name + "<- -"
  if cmp(p, "display name") == 0:
    value = name
    return value
  else:
    #convert the type passed by our caller
    #into the form AdminConfig uses within a configID
    p = p.lower() + "s"
  # step 2: split the string at the '|' and use everything BEFORE '|'
  #             Remember to remove the leading "("
  s = namelessString.split( "|" )
  #       DEBUG ONLY!!!!!
  #print "s.split( '|' ) - -> first string - ->" + s[0] + "<- -\n\n"
  # step 3: everything else is delimited by '/' so tokenize
  #           and take the token AFTER the one that matches p
  s[0] = s[0].replace( "("," " )
  s[0] = s[0].strip()
  s = s[0].split( "/" )
  for i in range( len(s) ):
    #       DEBUG ONLY!!!!!
    #print "s[",i,"] - - >" + s[i] + "< - -"
    if cmp(p, s[i]) == 0:
      value = s[i+1]
      #       DEBUG ONLY!!!!!
      #print p,"matches",s[i]
      break
    else:
      #       DEBUG ONLY!!!!!
      #print p,"does NOT match",s[i]
      pass
  return value

# findValueInObjectNameString
# parameters:
#   nameString - the value returned by AdminControl.queryNames
#   p - the property whose value we want to search for
# returns either the value or '"Not Found"
def findValueInObjectNameString( nameString, p ):
    """
If you provide the object name string of an MBean and the name of a property to find
in that string, this method parses the object name string and 
returns the value of that property.
parameters:
  nameString - the value returned by AdminControl.queryNames
  p - the property whose value we want to search for
returns either the value or '"Not Found"
    """
    returnValue = "Not Found"
    p += "="
    nameValuePairs = nameString.split( ',' )
    if len(nameValuePairs) > 1:
        for x in nameValuePairs:
            if x.find( p ) > -1:
                parts = x.split( '=' )
                returnValue = parts[1]
    return returnValue


# main method starts here
if __name__ == "main":
  print "Running standalone . . ."
else:
  print  __name__ , "module is loaded"
  #print __name__,"can see "
  #print dir()
