#
# threadMonitorConfigured.py
#

"""
 Methods in this module
 I.  Report the configuration of the hung thread monitor
 II. Modify the configuration of the hung thread monitor
     In order to modify, we need to know:
       1.  the server to modify
       2.  any non-default values for the hung thread monitor

"""
import AdminConfig, parsing


#   -  -  -  All global variables are meant to be READ ONLY -  -  -
# Global (IBM documented default) values (as of WAS 6.1)
# ___DefaultThreadCheckInterval = 180
# ___DefaultPresumptionOfHungExecutionTime = 600
# ___DefaultNumberOfFalseAlarmsAllowed = 100
# Global IBM property names
___IntervalPropertyName = "com.ibm.websphere.threadmonitor.interval"
___HungThresholdPropertyName ="com.ibm.websphere.threadmonitor.threshold"
___FalseAlarmPropertyName = "com.ibm.websphere.threadmonitor.false.alarm.threshold"
# Global - where will we find thread monitor values in the List we return
___IntervalElement = 0
___HungElement = 1
___FalseAlarmElement = 2
#Global Convenience Values
____BogusServerName = "```~~~!!!@@@"
____BogusNodeName = "```~~~!!!@@@"
____UnconfiguredDummyValue = -1
# misc Global variables
# ____IntervalDescription = ""
# ____HangTimeDescription = ""
# ____FalseAlarmDescription = ""
#   -  -  -  End of global (read only) variables area -  -  -

#
#getValues()
#
def getValues(serverName = ____BogusServerName, nodeName = ____BogusNodeName ):
  """
Returns the values of WebSphere Application Server's hung thread monitor service.  This method is meant to be called from other Python code.
parameters:
   serverName - mandatory
   nodeName - may be "" if the server name is unique within a cell
returns: a List of thread monitor values.  
            If it returns a List with every value set to -1, then something went wrong
            element #0 - monitor interval time.  Zero means IBM default
            element #1 - presumption of hung time.  Zero means IBM default
            element #2 - false alarm limit.  Zero means IBM default
  """
  ListOfThreadMonitorValues = [ ____UnconfiguredDummyValue, ____UnconfiguredDummyValue, ____UnconfiguredDummyValue ]
  if serverName.find( ____BogusServerName ) > -1:
    print "You must pass a valid server name"
    return ListOfThreadMonitorValues
  #the next line guarantees that we will return something useful
  #if any or all of our thread monitor values don't exist
  #print "DEBUG:  return values (initialized):  " + str( ListOfThreadMonitorValues )
  
  propertyList = [ ]
  
  #get the "components" attribute from this server
  containmentPath = ""
  containmentPath = __createServerContainmentPath( serverName, nodeName)
  configID = AdminConfig.getid( containmentPath )
  #print "DEBUG:  " + containmentPath
  #print "DEBUG:  " + configID
  #print "DEBUG:  return values at this point" + str( ListOfThreadMonitorValues )
  
  #educational comment:
  #internally, this appears to be an Xpath search of the configuration DOM tree
  #that returns a Python list serialized as a string
  rawComponents = AdminConfig.list( "Component", configID  )
  if len( rawComponents ) < 5:
    #if our raw string is this short, there probably aren't any Component elements
    print "No Components found in", configID
    print "rawComponents holds - - >" + rawComponents + "< - -"
    return ListOfThreadMonitorValues
  
  #do some work to turn the components into an List of components
  componentList = rawComponents.splitlines()
  
  #find the component that happens to be an ApplicationServer
  for x in componentList:
    #the component we want is the ApplicationServer component
    #print "DEBUG:  Component:"
    #print "DEBUG:  " + x
    if x.find( "#ApplicationServer_" ) > 0:
      #print "DEBUG:  #ApplicationServer_ component found"
      #does the ApplicationServer component have a "properties" attribute?
      #educational comment
      #this appears to be an Xpath search of a limited fragment
      #of the configuration DOM tree
      #that returns a TCL list serialized as a string
      result = AdminConfig.showAttribute( x, 'properties' )
      if len( result ) > 0:
        #print "DEBUG:  properties found:"
        #print "DEBUG:  " + result
        #get the string into a form we can use
        #by converting it to a Python list
        result = result.replace( '[', '' )
        result = result.replace( ']', '' )
        result = result.replace( ') ', '),' )
        propertyList = result.split(',')
        #if so, see if any properties are thread monitor properties
        for y in propertyList:
          if y.find( ___IntervalPropertyName ) > -1:
            ListOfThreadMonitorValues[___IntervalElement] = AdminConfig.showAttribute( y, 'value' )
          if y.find( ___HungThresholdPropertyName ) > -1:
            ListOfThreadMonitorValues[___HungElement] = AdminConfig.showAttribute( y, 'value' )
          if y.find( ___FalseAlarmPropertyName ) > -1:
            ListOfThreadMonitorValues[___FalseAlarmElement] = AdminConfig.showAttribute( y, 'value' )
        
  #return our findings to our caller
  return ListOfThreadMonitorValues

#
#setValues
#
def setValues(serverName = ____BogusServerName, nodeName = ____BogusNodeName, monitorIntervalTime = ____UnconfiguredDummyValue, presumptionOfHungTime = ____UnconfiguredDummyValue, falseAlarmLimit = ____UnconfiguredDummyValue ):
  """
Changes the values of WebSphere Application Server's hung thread monitor service
parameters:
   serverName - mandatory
   nodeName - may be "" if the server name is unique within a cell
   monitorIntervalTime.  Set to zero to restore IBM default
   presumptionOfHungTime.  Set to zero to restore IBM default
   falseAlarmLimit.  Set to zero to restore IBM default
returns:
   nothing
  """
  if serverName.find( ____BogusServerName ) > -1:
    print "You must pass a valid server name"
    return
  #get the "components" attribute from this server
  containmentPath = ""
  containmentPath = __createServerContainmentPath( serverName, nodeName)
  configID = AdminConfig.getid( containmentPath )
  #educational comment:
  #internally, this appears to be an Xpath search of the configuration DOM tree
  #that returns a Python list serialized as a string
  rawComponents = AdminConfig.list( "Component", configID  )
  if len( rawComponents ) < 5:
    #if our raw string is this short, we failed
    return ListOfThreadMonitorValues
  
  #do some work to turn the components into an List of components
  componentList = rawComponents.splitlines()
  
  #find the component that happens to be an ApplicationServer
  for x in componentList:
    #the component we want is the ApplicationServer component
    if x.find( "#ApplicationServer_" ) > 0:
      #remove any existing properties
      AdminConfig.modify( x, [ [ 'properties', [] ] ] )
      #replace them with our values
      modifyList = []
      propertiesList = ['properties']
      innerList = []
      scrap = []
      if  monitorIntervalTime > 0:
        scrap = [ ['name',___IntervalPropertyName ], [ 'value', monitorIntervalTime ] ]
        innerList.append( scrap )
        scrap = []
      if  presumptionOfHungTime > 0:
        scrap = [ [ 'name',___HungThresholdPropertyName ], [ 'value', presumptionOfHungTime ] ]
        innerList.append( scrap )
        scrap = []
      if  falseAlarmLimit > 0:
        scrap = [ [ 'name',___FalseAlarmPropertyName ], [ 'value', falseAlarmLimit ] ]
        innerList.append( scrap )
        scrap = []
      propertiesList.append( innerList )
      modifyList.append( propertiesList )
      #THREE lines of DEBUG prints follow
      #print "\nDEBUG:  About to modify " + x
      #print "using " + str( modifyList )
      #print "\n\n"
      AdminConfig.modify( x, modifyList )

#
#getAll
#
def reportAll():
  """
Finds every server in a cell and reports the ThreadMonitor configuration -- if any -- for each server.  This report is printed on the screen.  It is meant for human consumption rather than use by other python code.
returns:
    nothing
  """
  raw = AdminConfig.list( 'Server' )
  allServers = raw.splitlines()
  print "\n\nAny value that has not been configured is displayed as " + str( ____UnconfiguredDummyValue )
  for server in allServers:
    s = parsing.findValueInConfigurationIDString( server, 'server')
    n = parsing.findValueInConfigurationIDString( server, 'node' )
    valueList = getValues( s, n )
    print "\nFor " + s + " in Node " + n
    print "the *configured* values for the WAS Thread Monitor are:"
    print valueList
    print ___IntervalPropertyName + " is " + str( valueList[___IntervalElement] )
    print ___HungThresholdPropertyName + " is " + str( valueList[___HungElement]  )
    print ___FalseAlarmPropertyName + " is " + str( valueList[___FalseAlarmElement] )
  return


#    PRIVATE FUNCTION -- for internal use only!!!
# __createServerContainmentPath
#Creates the containment string that we need in order to use the various
#AdminConfig methods
# returns:
#    a containment string that may or may not work
# parameters:
#    serverName - mandatory
#    nodeName - may be "" if the server name is unique within a cell
def __createServerContainmentPath( serverName, nodeName):
  if len(nodeName) < 1:
    return "/Server:" + serverName + "/"
  if nodeName.find( ____BogusNodeName ) > -1:
    return "/Server:" + serverName + "/"
  return "/Node:" + nodeName + "/Server:" + serverName + "/"




# the real main function code starts here
# the only purpose of this code is 
# to provide confirmation that the module loaded
if __name__ == "main":
  print "threadMonitorConfigured is running standalone . . ."
else:
  print  __name__ , "module is loaded"
  #print __name__,"can see "
  #print dir()
