#
# threadMonitorRuntime.py
#
"""
Methods in this module intorrogate MBeans in order to:
I.  Report the current parameters of the hung thread monitor of a running server.
II. Modify the current parameters of the hung thread monitor of a running server.
In order to modify, we need to know:
    1.  the server to modify (so that we can find the MBean that controls the behavour of that server)
    2.  any non-default values for the hung thread monitor
"""

import AdminControl, parsing

#   -  -  -  All global variables are meant to be READ ONLY -  -  -
# Global (IBM default) values
# _____DefaultThreadCheckInterval = 180
# _____DefaultPresumptionOfHungExecutionTime = 600
# _____DefaultNumberOfFalseAlarmsAllowed = 100
# Global IBM property names
_____IntervalPropertyName = "com.ibm.websphere.threadmonitor.interval"
_____HungThresholdPropertyName ="com.ibm.websphere.threadmonitor.threshold"
_____FalseAlarmPropertyName = "com.ibm.websphere.threadmonitor.false.alarm.threshold"
# Global - where will we find thread monitor values in the List we return
_IntervalElement = 0
_HungElement = 1
_FalseAlarmElement = 2
#Global Convenience Values
_BogusServerName = "```~~~!!!@@@"
_BogusNodeName = "```~~~!!!@@@"
_UnconfiguredDummyValue = -1
# misc Global variables
# __IntervalDescription = ""
# __HangTimeDescription = ""
# __FalseAlarmDescription = ""
#   -  -  -  End of global (read only) variables area -  -  -


#
#getValues()
#
def getValues(serverName = _BogusServerName, nodeName = _BogusNodeName ):
  """
Get a list of parameters that control the behavour of the ThreadMonitor for one server.
It is an error to give this method parameters that match more than one MBean
parameters:
   serverName - mandatory
   nodeName - may be "" if the server name is unique within a cell
returns: a List of thread monitor values for one server.  
            If it returns a List with every value set to -1, then something went wrong
            element #0 - monitor interval time.  Zero means IBM default
            element #1 - presumption of hung time.  Zero means IBM default
            element #2 - false alarm limit.  Zero means IBM default
  """
  ListOfThreadMonitorValues = [ _UnconfiguredDummyValue, _UnconfiguredDummyValue, _UnconfiguredDummyValue ]
  if serverName.find( _BogusServerName ) > -1:
    print "You must pass a valid server name"
    return ListOfThreadMonitorValues
  nameString = __createServerNameString( serverName, nodeName )
  mBeanName = AdminControl.queryNames( nameString )
  ListOfThreadMonitorValues[_IntervalElement] = AdminControl.invoke( mBeanName, 'getThreadMonitorInterval' )
  ListOfThreadMonitorValues[_HungElement] = AdminControl.invoke( mBeanName, 'getThreadMonitorThreshold' )
  ListOfThreadMonitorValues[_FalseAlarmElement] = AdminControl.invoke( mBeanName, 'getThreadMonitorAdjustmentThreshold' )
  return ListOfThreadMonitorValues


#
#setValues
#
def setValues(serverName = _BogusServerName, nodeName = _BogusNodeName, monitorIntervalTime = _UnconfiguredDummyValue, presumptionOfHungTime = _UnconfiguredDummyValue, falseAlarmLimit = _UnconfiguredDummyValue ):
  """
Change the parameters that control the behavour of the ThreadMonitor for one server.
It is an error to give this method parameters that match more than one MBean
parameters:
   serverName - mandatory
   nodeName - may be "" if the server name is unique within a cell
   monitorIntervalTime.  Set to zero to restore IBM default
   presumptionOfHungTime.  Set to zero to restore IBM default
   falseAlarmLimit.  Set to zero to restore IBM default
returns:
   nothing

  """
  if serverName.find( _BogusServerName ) > -1:
    print "You must pass a valid server name"
    return
  nameString = __createServerNameString( serverName, nodeName )
  mBeanName = AdminControl.queryNames( nameString )
  if monitorIntervalTime > 0:
    AdminControl.invoke( mBeanName, 'setThreadMonitorInterval', str(monitorIntervalTime) )
  if presumptionOfHungTime > 0:
    AdminControl.invoke( mBeanName, 'setThreadMonitorThreshold', str(presumptionOfHungTime) )
  if falseAlarmLimit > 0:
    AdminControl.invoke( mBeanName, 'setThreadMonitorAdjustmentThreshold', str(falseAlarmLimit) )
  return


#
#reportAll
#
def reportAll():
  """
Intorrogates the MBean in front of every running server in a cell and reports the actual current ThreadMonitor values for each server.
This function is meant for human consumption.  It is not meant to be called by Python code.
returns:
   nothing
  """
  valueList = [ _UnconfiguredDummyValue, _UnconfiguredDummyValue, _UnconfiguredDummyValue ]
  raw = AdminControl.queryNames( 'type=Server,*' )
  
  #TWO lines of DEBUG prints
  #print "DEBUG:  these are the servers that are running right now in our cell"
  #print raw
  
  print "\n"
  allServers = raw.splitlines()
  for server in allServers:
    s = parsing.findValueInObjectNameString( server, 'process')
    n = parsing.findValueInObjectNameString( server, 'node' )
    
    #FOUR lines of DEBUG prints
    #print "DEBUG:  server:  "
    #print server
    #print "DEBUG:  s:  " + s
    #print "DEBUG:  n:  " + n
    
    valueList = getValues( s, n )
    print "\nFor " + s + " in Node " + n
    print "the actual current values for the WAS Thread Monitor are:"
    print valueList
    print _____IntervalPropertyName + " is " + str( valueList[_IntervalElement] )
    print _____HungThresholdPropertyName + " is " + str( valueList[_HungElement]  )
    print _____FalseAlarmPropertyName + " is " + str( valueList[_FalseAlarmElement] )
  return


#    PRIVATE FUNCTION -- for internal use only!!!
# __createServerNameString
#Creates the name string that we need in order to use the various AdminControl methods
# returns:
#    a name string that may or may not work
# parameters:
#    serverName - mandatory
#    nodeName - may be "" if the server name is unique within a cell
def __createServerNameString( serverName, nodeName):
  if len(nodeName) < 1:
    return "type=Server,name=" + serverName + ",*"
  if nodeName.find( _BogusNodeName ) > -1:
    return "type=Server,name=" + serverName + ",*"
  return "type=Server,node=" + nodeName + ",name=" + serverName + ",*"






# the real main function code starts here
# the only purpose of this code is 
# to provide confirmation that the module loaded
if __name__ == "main":
  print "threadMonitorRuntime is running standalone . . ."
else:
  print  __name__ , "module is loaded"
  #print __name__,"can see "
  #print dir()
